// File: Main.pas
//
// Description: Main application form demonstrating virtual
// printer transfer mode related configuration

unit Main;

interface

uses Windows, SysUtils, Classes, Controls, Forms, StdCtrls, ExtCtrls, Settings;

type
  // Main form class
  TMainForm = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    ClientAppEdit: TEdit;
    ExeBrowseButton: TButton;
    Label2: TLabel;
    WndClassEdit: TEdit;
    Label3: TLabel;
    WndTitleEdit: TEdit;
    Label4: TLabel;
    ClipboardFmtEdit: TEdit;
    Label5: TLabel;
    MsgIdEdit: TEdit;
    Label6: TLabel;
    TransferModeComboBox: TComboBox;
    Label7: TLabel;
    OutputPathEdit: TEdit;
    DirBrowseButton: TButton;
    GroupBox2: TGroupBox;
    EmfCheckBox: TCheckBox;
    BmpCheckBox: TCheckBox;
    PngCheckBox: TCheckBox;
    JpegCheckBox: TCheckBox;
    TiffCheckBox: TCheckBox;
    PdfCheckBox: TCheckBox;
    TxtCheckBox: TCheckBox;
    RedirectCheckBox: TCheckBox;
    LanguageCheckBox: TCheckBox;
    SaveButton: TButton;
    CloseButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ExeBrowseButtonClick(Sender: TObject);
    procedure DirBrowseButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
  private
    FPrnSettings: TPrnSettings;
  end;

var
  MainForm: TMainForm;

implementation

uses SHDialogs;

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.Title := Caption;

  // Load settings from the registry
  FPrnSettings := TPrnSettings.Create;
  if not FPrnSettings.Load then
    MessageBox(Handle, 'Can not load printer settings from the registry',
      'Configuration', MB_ICONERROR or MB_OK);

  // Set up the controls
  ClientAppEdit.Text := FPrnSettings.ClientAppPath;
  WndClassEdit.Text := FPrnSettings.ClientWndClass;
  WndTitleEdit.Text := FPrnSettings.ClientWndTitle;
  ClipboardFmtEdit.Text := FPrnSettings.ClipboardFormat;
  MsgIdEdit.Text := Format('%u (0x%.8X)', [FPrnSettings.MessageId,
    FPrnSettings.MessageId]);
  TransferModeComboBox.ItemIndex := Integer(FPrnSettings.TransferMode);
  OutputPathEdit.Text := FPrnSettings.OutputPath;

  EmfCheckBox.Checked := not FPrnSettings.HideEmfPage;
  BmpCheckBox.Checked := not FPrnSettings.HideBmpPage;
  PngCheckBox.Checked := not FPrnSettings.HidePngPage;
  JpegCheckBox.Checked := not FPrnSettings.HideJpegPage;
  TiffCheckBox.Checked := not FPrnSettings.HideTiffPage;
  PdfCheckBox.Checked := not FPrnSettings.HidePdfPage;
  TxtCheckBox.Checked := not FPrnSettings.HideTxtPage;
  RedirectCheckBox.Checked := not FPrnSettings.HideRedirectPage;
  LanguageCheckBox.Checked := not FPrnSettings.HideLanguagePage;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  FPrnSettings.Destroy;
end;

procedure TMainForm.ExeBrowseButtonClick(Sender: TObject);
var
  s: string;
begin
  if OpenFileDialogBox(Handle, s) then
    ClientAppEdit.Text := s;
end;

procedure TMainForm.DirBrowseButtonClick(Sender: TObject);
var
  s: string;
begin
  if OpenFolderDialogBox(Handle, s) then
    OutputPathEdit.Text := s;
end;

procedure TMainForm.SaveButtonClick(Sender: TObject);
var
  MsgId: Longword;
  i: Integer;
  s: string;
begin
  // Get Message ID
  s := MsgIdEdit.Text;
  i := LastDelimiter(' (', s);
  if i > 0 then s := Copy(s, 1, i-1);
  Val(s, MsgId, i);
  MsgIdEdit.Text := Format('%u (0x%.8X)', [MsgId, MsgId]);

  // Check Message ID
  if (TransferModeComboBox.ItemIndex = Integer(tmThroughClipboard)) and
    ((MsgId < $8000) or (MsgId > $BFFF)) then
  begin
    MessageBox(Handle, 'Message ID for the selected Transfer mode'#13#10 +
      'must be in range from 32768 to 49151', 'Configuration', MB_ICONWARNING or MB_OK);
    Exit;
  end;

  // Update settings in CPrnSettings object
  FPrnSettings.ClientAppPath := ClientAppEdit.Text;
  FPrnSettings.ClientWndClass := WndClassEdit.Text;
  FPrnSettings.ClientWndTitle := WndTitleEdit.Text;
  FPrnSettings.ClipboardFormat := ClipboardFmtEdit.Text;
  FPrnSettings.MessageId := MsgId;
  FPrnSettings.TransferMode := TTransferModes(TransferModeComboBox.ItemIndex);
  FPrnSettings.OutputPath := OutputPathEdit.Text;

  FPrnSettings.HideEmfPage := not EmfCheckBox.Checked;
  FPrnSettings.HideBmpPage := not BmpCheckBox.Checked;
  FPrnSettings.HidePngPage := not PngCheckBox.Checked;
  FPrnSettings.HideJpegPage := not JpegCheckBox.Checked;
  FPrnSettings.HideTiffPage := not TiffCheckBox.Checked;
  FPrnSettings.HidePdfPage := not PdfCheckBox.Checked;
  FPrnSettings.HideTxtPage := not TxtCheckBox.Checked;
  FPrnSettings.HideRedirectPage := not RedirectCheckBox.Checked;
  FPrnSettings.HideLanguagePage := not LanguageCheckBox.Checked;

  // Save settings to the registry
  if FPrnSettings.Save then
    Close
  else
    MessageBox(Handle, 'Can not save printer settings to the registry',
      'Configuration', MB_ICONERROR or MB_OK);
end;

procedure TMainForm.CloseButtonClick(Sender: TObject);
begin
  Close;
end;

end.
