// File: SHDialogs.pas
//
// Description: Special dialog functionality
// (.exe-file and folder selection dialogs)

unit SHDialogs;

interface

uses Windows, SysUtils, ShlObj, CommDlg, ActiveX;

// This function opens OpenFile dialog and returns full path to the
// selected filename in FileName parameter; function will return
// false if file selection will be cancelled
function OpenFileDialogBox(const Wnd: HWND; var FileName: string): Boolean;

// This function opens OpenFolder dialog and returns the name of
// the selected folder in FolderName parameter; function will
// return false if folder selection will be cancelled
function OpenFolderDialogBox(const Wnd: HWND; var FolderName: string): Boolean;

implementation

function SHBindToParent(pidl: PItemIDList; const riid: TIID; out ppvOut;
  out ppidlLast: PItemIDList): HRESULT; stdcall; external 'shell32.dll';

// This function opens OpenFile dialog and returns full path to the
// selected filename in FileName parameter; function will return
// false if file selection will be cancelled
function OpenFileDialogBox(const Wnd: HWND; var FileName: string): Boolean;
var
  Par: TOpenFileName;
  Buf: array[0..MAX_PATH - 1] of Char;
begin
  FillChar(Par, SizeOf(Par), 0);
  Par.lStructSize := SizeOf(Par);
  Par.Flags := OFN_EXPLORER or OFN_FILEMUSTEXIST or OFN_HIDEREADONLY;
  Par.lpstrFilter := '.exe files (*.exe)'#0'*.exe'#0'All files (*.*)'#0'*.*'#0;
  Par.lpstrFile := @Buf;
  Par.nMaxFile := MAX_PATH;
  Par.lpstrDefExt := 'exe';
  Par.hwndOwner := Wnd;

  FileName := ''; Buf[0] := #0;
  Result := GetOpenFileName(Par);
  if Result then FileName := PChar(@Buf);
end;

// This function retrieves the UIObject interface for the specified full PIDL
function SHGetUIObjectFromFullPIDL(pidl: PItemIDList; hwndOwner: HWND;
  const riid: TIID; var ppvOut): HRESULT;
var
  pidlChild: PItemIDList;
  psf: IShellFolder;
begin
  Pointer(ppvOut) := nil;

  Result := SHBindToParent(pidl, IID_IShellFolder, psf, pidlChild);
  if SUCCEEDED(Result) then
  begin
    Result := psf.GetUIObjectOf(hwndOwner, 1, pidlChild, riid, nil, ppvOut);
    psf := nil;
  end;
end;

function SHILClone(pidl: PItemIDList; var ppidl: PItemIDList): HRESULT;
var
  cbTotal: DWORD;
  pidl_temp: PItemIDList;
begin
  cbTotal := 0;

  if Assigned(pidl) then
  begin
    pidl_temp := pidl;
    Inc(cbTotal, SizeOf(pidl_temp^.mkid.cb));
    while (pidl_temp^.mkid.cb <> 0) do
    begin
      Inc(cbTotal, pidl_temp^.mkid.cb);
      pidl_temp := Pointer(Cardinal(pidl_temp) + pidl_temp^.mkid.cb);
    end;
  end;
  ppidl := CoTaskMemAlloc(cbTotal);
  if Assigned(ppidl) then CopyMemory(ppidl, pidl, cbTotal);

  if Assigned(ppidl) then Result := S_OK else
    Result := E_OUTOFMEMORY;
end;

// This function gets the target PIDL for a folder PIDL. This deals with cases
// where a folder is an alias to a real folder, folder shortcuts, etc.
function SHGetTargetFolderIDList(pidlFolder: PItemIDList;
  var ppidl: PItemIDList): HRESULT;
var
  psl: IShellLink;
begin
  ppidl := nil;

  Result := SHGetUIObjectFromFullPIDL(pidlFolder, 0, IID_IShellLinkA, psl);

  if SUCCEEDED(Result) then
  begin
    Result := psl.GetIDList(ppidl);
    psl := nil;
  end else
    // It's not a folder shortcut so get the PIDL normally
    Result := SHILClone(pidlFolder, ppidl);
end;

// This function opens OpenFolder dialog and returns the name of
// the selected folder in FolderName parameter; function will
// return false if folder selection will be cancelled
function OpenFolderDialogBox(const Wnd: HWND; var FolderName: string): Boolean;
var
  Par: TBrowseInfo;
  pidl, pidlTarget: PItemIDList;
  Buf: array[0..MAX_PATH - 1] of Char;
begin
  FillChar(Par, SizeOf(Par), 0);
  Par.lpszTitle := 'Select new folder for generated files:';
  Par.ulFlags := BIF_DONTGOBELOWDOMAIN or BIF_RETURNONLYFSDIRS;
  Par.hwndOwner := Wnd;

  FolderName := '';
  CoInitializeEx(nil, COINIT_APARTMENTTHREADED);
  pidl := SHBrowseForFolder(Par);

  if Assigned(pidl) then
  begin
    if SUCCEEDED(SHGetTargetFolderIDList(pidl, pidlTarget)) then
    begin
      SHGetPathFromIDList(pidlTarget, @Buf);
      CoTaskMemFree(pidlTarget);
      FolderName := PChar(@Buf);
    end;
    CoTaskMemFree(pidl);
  end;

  Result := Assigned(pidl);
end;

end.
